LBL <-
  function(data.ex, baseline = "missing",allelic = T, a = 5, b = 5, start.beta = 0.01, lambda = 1, D = 0, seed = NULL, e = 0.1, burn.in = 10000, num.it = 40000) 
  {
    if (is.null(seed)==FALSE) set.seed(seed)
    
    if (!"hapassoc" %in% installed.packages())
    {
      cat("Installing required package hapassoc...\n\n")
      install.packages("hapassoc", repos = "http://cran.us.r-project.org", quiet = T)
      cat("Done\n\n")}
    
    require(hapassoc)
    
    n.fam <- length(unique(data.ex[,1]))
    aff <- data.ex[data.ex[,3]!=0 & data.ex[,4]!=0 & data.ex[,6]==2,]
    fa <- data.ex[match(aff[,3],data.ex[,2]),]
    mo <- data.ex[match(aff[,4],data.ex[,2]),]
    data.new <- rbind(fa[,-(1:5)], mo[,-(1:5)],aff[,-(1:5)])
        
    if (allelic)  {
      p <- (ncol(data.new)-1)/2
    } else p <- ncol(data.new) - 1
    
    haplos.list <- pre.hapassoc(data.new, p, pooling.tol = 0, allelic = allelic)
    haplos.names <- names(haplos.list$initFreq)
    freq <- haplos.list$initFreq
    ID <- haplos.list$ID
    N <- n.fam*2
    n <- n.fam
    x <- data.matrix(haplos.list$haploDM)
    
    nonunique <- unique(ID[grep(T, haplos.list$wt != 1)])
    nonuni.fa <- nonunique[nonunique <= n]
    nonuni.mo <- nonunique[nonunique > n & nonunique <= 2*n]
    nonuni.kid <- nonunique[nonunique > 2*n] 
    nonuni.family <- sort(unique(c(nonuni.fa, (nonuni.mo - n), (nonuni.kid - 2*n))))
    
    xc <- xu <- NULL
    count <- 0
    num.haplo.id <- rep(1, n)
    for ( i in 1:n) {
      if (i %in% nonuni.family){
        for (j in grep(T, ID==i)) {
          for (k in grep(T, ID==(i+n))) {
            for (m in grep(T, ID == (i + 2*n))) {
              child.hap <- grep(FALSE, x[m,] == 0)
              if (length(child.hap) ==1) child.hap = rep(child.hap, 2)
              judge.father <- sapply(child.hap, '%in%', grep(FALSE, x[j,] == 0))
              if (sum(judge.father) == 2) {
                judge <- sum(sapply(child.hap, '%in%', grep(FALSE, x[k,] == 0))) > 0
              } else if (sum(judge.father) == 1) {
                judge <- child.hap[grep(FALSE, judge.father)] %in% grep(FALSE, x[k,] == 0)
              } else if (sum(judge.father) == 0) {
                judge <- 0
              } else stop("error in configuring family haplotype!")
              
              if (judge > 0) {
                tmp <- x[j,] + x[k,] - x[m,]
                xc <- rbind(xc, x[m,])
                xu <- rbind(xu, tmp)
                count <- count + 1
              }
            }
          }
        }
        num.haplo.id[i] <- count
        count <- 0
      } else { 
        xc <- rbind(xc, x[ID == (i + 2*n),])
        xu <- rbind(xu, (x[ID == i,] + x[ID == (i + n),] - x[ID == (i + 2*n),]))
      }
    }	
    
    
    if (any(num.haplo.id==0)) {
      cat("Incompatile trios have been detected, and will be removed from analysis!\n")
      num.haplo.id <- num.haplo.id[num.haplo.id !=0]
      N <- length(num.haplo.id)*2
      cat("A total of", length(num.haplo.id), "families remain\n")
    }
    
    x <- rbind(xc, xu)
    rownames(x) <- NULL
    
    if (!(baseline %in% colnames(haplos.list$haploDM))&baseline != "missing") 
    {
      warning("Baseline haplotype does not exist!")
      baseline<-"missing"
    }
    
    if (baseline=="missing")
    {
      baseline <- haplos.names[which.max(freq)]
    }
    
    column.subset <- colnames(haplos.list$haploDM) != baseline
    x <- data.matrix(x[,column.subset])
    y <- rep(1:0, c(nrow(xc), nrow(xu)))
    
    freq.new<-freq[column.subset]
    freq.new[length(freq.new)+1]<-freq[!column.subset]
    freq.new<-as.vector(freq.new)
    x.length<-as.integer(dim(x)[2])
    unique.x<-unique(x)	
    
    beta=rep(start.beta, x.length)
    beta.out<-numeric((num.it-burn.in)*x.length)
    lambda.out<-numeric(num.it-burn.in)
    freq.out<-numeric((num.it-burn.in)*(x.length+1))
    D.out<-numeric(num.it-burn.in)
    #dyn.load("famLBL.so")
    #dyn.load("BL_fam.so")
    num.haplo.id <- rep(num.haplo.id,2)   
    
    out<-.C("mcmc", x=as.integer(x), n=as.integer(dim(x)[1]), as.integer(y), as.integer(N), as.integer(num.haplo.id), as.integer(x.length), as.double(freq.new), as.double(D), as.double(beta), as.double(a), as.double(b), as.integer(t(unique.x)), as.integer(dim(unique.x)[1]), as.double(lambda), as.integer(num.it), as.integer(burn.in), beta.out=as.double(beta.out), lambda.out=as.double(lambda.out), freq.out=as.double(freq.out), D.out=as.double(D.out))
    
    beta.out<-matrix(out$beta.out,nrow=num.it-burn.in, byrow=T)
    
    #OR estimates and CIs
    beta.ci <- exp(apply(beta.out, 2, quantile, c(0.025, 0.975)))
    haplo.names <- names(haplos.list$initFreq)[column.subset]
    haplo.names <- c(haplos.names, names(haplos.list$initFreq)[!column.subset])
    beta.est <- exp(colMeans(beta.out))
    
    #frequency estimates and CIs, without the baseline haplotype
    freq.out <- matrix(out$freq.out, nrow=num.it-burn.in, byrow=T)
    #    freq.out <- freq.out[, -ncol(freq.out)]
    freq.est <- colMeans(freq.out)
    freq.ci <- apply(freq.out, 2, quantile, c(0.025, 0.975))
    
    lambda.ci <- quantile(out$lambda.out, c(0.025, 0.975))
    d.ci <- quantile(out$D.out, c(0.025, 0.975))
    
    #calculating the Bayes Factors
    prob.alt<-numeric(x.length)
    BF<-rep(999, x.length)
    
    prior.prob<-(b/(e+b))^a
    for (i in 1:x.length)
    {
      prob.alt[i]<-length(beta.out[,i][abs(beta.out[,i]) > e])/(num.it-burn.in)
      if (prob.alt[i] < 1) BF[i]<-(prob.alt[i]/(1-prob.alt[i]))/(prior.prob/(1-prior.prob)) 
    }
    BF[BF==999] <-">100"
    
    result <- list(haplotypes=haplo.names, freq=freq.est, freq.CI = t(freq.ci), OR=beta.est, OR.CI=t(beta.ci), BF=BF, lambda.CI = lambda.ci, D.ci = d.ci)
    
    return(result)
  }
