\name{LBL}
\alias{LBL}
%- Also NEED an '\alias' for EACH other topic documented here.
\title{
Bayesian Lasso for Familywise Haplotype Association Studies
}
\description{
This function takes a dataset of families with multiple SNP genotypes and a binary pheonotype. Bayesian lasso is used to find the posterior
     distributions of logistic regression coefficients, which are then
     used to calculate Bayes Factor to test for association with haplotypes.
     Parts of code is borrowed from the hapassoc function in hapassoc
     package as well the arguments and values.
}
\usage{
LBL(data, baseline = "missing", allelic = T, a = 5, b = 5, start.beta = 0.01, lambda = 1, D = 0, seed = NULL, e = 0.1, burn.in = 10000, num.it = 40000)
}
%- maybe also 'usage' for other objects documented here.
\arguments{
  \item{data}{
Data is in typical family data format. The data for each individual occupy
one row, starting with family id, individual id, father id, mother id, 
followd by sex (1=male, 2=female) and affection status (1=unaffected, 2=affected). The remaining of the columns code for SNP genotypes. See "allelic" below for 
detailed description.
}
  \item{baseline}{
optional, haplotype to be used for baseline coding. Default
    is the most frequent haplotype according to the initial
    haplotype frequency estimates returned by pre.hapassoc.
}
  \item{allelic}{TRUE if single-locus SNP genotypes are in allelic format and FALSE if in genotypic format; default is TRUE
}
  \item{a}{
first hyperparameter of the prior for regression coefficients, beta. The
  prior variance of beta is 2/lambda^2 and lambda has Gamma(a,b)
  prior. The Gamma parameters a and b are such that the mean and
  variance of the Gamma distribution are a/b and a/b^2.
}
  \item{b}{
  b parameter of the Gamma(a,b) distribution described above.
}
  \item{start.beta}{
starting value of all regression coefficients, beta.
}
  \item{lambda}{
starting value of the lambda parameter described above.
}
  \item{D}{
starting value of the D parameter, which is the within-population
     inbreeding coefficient.

}
  \item{seed}{
the seed to be used for the MCMC in Bayesian Lasso.
}
  \item{e}{
A (small) number epsilon in the null hypothesis of no association,
    H0: |beta| <= epsilon. Changing e from default of 0.1 may need choosing a
     different threshold for Bayes Factor (one of the outputs) to
     infer association.
}
  \item{burn.in}{
Burn-in period of the MCMC sampling scheme.
}
  \item{num.it}{
Total number of MCMC iterations including the burn-in iterations.
}
}
\details{
  Starting value of haplotype frequencies are the estimates obtained
  from pre.hapassoc.
}
\value{
  \item{haplotype }{Haplotypes esimated from pre.hapassoc. The last haplotype is the baseline haplotype.}
  \item{frequency }{A vector of estimated frequencies of the corresponding haplotypes.}
  \item{frequency.ci }{95\% credible sets for the estimated frequencies.}
  \item{OR }{A vector of estimated odds ratios of the corresponding haplotype against the reference haplotype (baseline). This is the exponentoal of the posterior means of the regression coefficients.}
  \item{OR.ci }{95\% credible sets for the estimated odds ratios.}
  \item{BF }{A vector of Bayes Factors for all regression
    coefficients. If BF exceeds a cetain threshold (e.g., 2 or 3)
    association can be concluded.}
  \item{lamda.ci }{95\% credible set for lambda.}
  \item{D.ci }{95\% credible sets for the inbreeding coefficient, D.}
  
}
\references{
Biswas S, and Lin, S. (2011) Logistic Bayesian Lasso for identifying association with
 rare haplotypes and application to age-related macular degeneration. Manuscript.

Burkett K, Graham J and McNeney B (2006). hapassoc: Software for
     Likelihood Inference of Trait Associations with SNP Haplotypes and
     Other Attributes. Journal of Statistical Software, *16(2)*:1-19
}
\author{
Swati Biswas, Shili Lin
}
\note{
%%  ~~further notes~~
}

%% ~Make other sections like Warning with \section{Warning }{....} ~

\seealso{
'pre.hapassoc', 'hapassoc', 'rGLM'
}
\examples{
# Load an example data
data(data.ex)

# Run BL to make inference on association of haplotypes with the disease. Note the defaul setting are burn.in = 20000, num.it = 50000 in the BL function. However, you may want to use smaller numbers for a quick check to make sure the package is loded properly.
out.BL<-LBL(data.ex, burn.in=100, num.it=1000)
}

